.global _loader           # making entry point visible to linker

# setting up the Multiboot header - see GRUB docs for details
.set ALIGN,    1<<0             # align loaded modules on page boundaries
.set MEMINFO,  1<<1             # provide memory map
.set FLAGS,    ALIGN | MEMINFO  # this is the Multiboot 'flag' field
.set MAGIC,    0x1BADB002       # 'magic number' lets bootloader find the header
.set CHECKSUM, -(MAGIC + FLAGS) # checksum required

.align 4
.long MAGIC
.long FLAGS
.long CHECKSUM

# reserve initial kernel stack sace
.set STACKSIZE, 0x4000          # that's 16k.
.comm stack, STACKSIZE, 32      # reserve 16k stack on a quadword boundary

_loader:       mov   $(stack + STACKSIZE), %esp # set up the stack
               push  %eax                       # Multiboot magic number
               push  %ebx                       # Multiboot data structure

	       lgdt	(gdt_ptr)	
		
	    mov 	$0x10, %ax
	    mov		%eax, %ds
	    mov		%eax, %es
	    mov		%eax, %ss
	    mov		%eax, %fs
	    mov		%eax, %gs

	    ljmp	$0x08,$flush
		
flush:


	    movw	$0x9F44,(0xB8000) 

	    
	    
	    
	    mov		$idt_end , %ecx
	    sub		$idt, %ecx    
	    shr		$3, %ecx
	    
#	    mov 	$256, %ecx

	    mov		$idt, %edi
	    mov		$henkel, %eax
_loop:
	    push	%eax
	    mov		%ax, (%edi)	    
	    shr		$16, %eax
    
	    mov 	%ax, 6(%edi)
	    add		$8, %edi
    	    pop		%eax
loop	_loop


	    mov		$timer, %eax
	    movw	%ax, (idt+256)
	    shrl	$16, %eax
	    movw	%ax, (idt+256+6)
	    movb	$0x8e, (idt+256+5)




	    mov		$hrm, %eax
	    movw	%ax, (idt+256+8)
	    shrl	$16, %eax
	    movw	%ax, (idt+256+6+8)
	    movb	$0x8e, (idt+256+5+8)

	    lidt	idt_ptr     

	     

	#    jmp .
             sti


	       call  _main            # call kernel proper

	       hlt                    # halt machine should kernel return
	
	
hrm:

	    pusha
	    push %ds
	    push %es
	    push %fs
	    push %gs
    
	    mov		$0x10,%ax
	    mov		 %ax, %ds
    
#	    mov		$0xb8000, %edi

#	    mov 	$0x60, %dx
#	    in		%dx,%al
#	    add		$0x4141, %eax
#	    mov 	$0xff60, %eax
#	    stosl
#	    jne		urk
#	    mov		$0xabff, %eax

	    
#	    rep		movsl	    	
urk:
	
#	    movw	$0xabFF,(0xB8000)
#	    inc		%edi
#	    movw	$0x94FF, %ds:0xb8000

	call key_isr    
	#    call touchme	    
#	    hlt
	    
	    mov	$0x20, %al
	    out %al, $0x20 

	    pop %gs
	    pop %fs
	    pop %es
	    pop %ds
	    popa
	
	    iret
    # hlt
    
#	    hlt
#	    ljmp .


henkel:

	    pusha
	    push %ds
	    push %es
	    push %fs
	    push %gs
    
#	    mov	$20, %al
#	    out %al, $20 
	    mov $10, %ax
#	    mov %ax,%ds
	    	    
#	    call key_isr

	    mov	$0x20, %al
	    out %al, $0x20 

	    pop %gs
	    pop %fs
	    pop %es
	    pop %ds
	    popa

	    iret
		    
timer:
	    
	    pusha
#	    movl	$0xfff, 3+(0xb8000)
	    call timer_isr
	    mov	$0x20, %al
	    out %al, $0x20
	    popa
	    iret



#.section data
#.equ hmfh, hrm >> 16
.data

gdt:
# NULL descriptor
	.word 	0		# limit 15:0
	.word 	0		# base 15:0
	.byte 	0		# base 23:16
	.byte 	0		# type
	.byte 	0		# limit 19:16, flags
	.byte 	0		# base 31:24

# LINEAR_CODE_SEL: .word	.-gdt
	.word	 0xFFFF
	.word	 0
	.byte	 0
	.byte	 0x9A		# present,ring 0,code,non-conforming,readable
	.byte	 0xCF		# page-granular (4 gig limit), 32-bit
	.byte	 0

# LINEAR_DATA_SEL: .word .-gdt
	.word	 0xFFFF
	.word	 0
	.byte	 0
	.byte	 0x92		# present, ring 0, data, expand-up, writable
	.byte	 0xCF		# page-granular (4 gig limit), 32-bit
	.byte	 0
gdt_end:

gdt_ptr:
	.word 	 gdt_end - gdt - 1
	.long	 gdt


idt:
.rept 0x256
	.word	0		# offset 15:0
	.word	0x8		# GDT selector
	.byte	0		# Always Zero for interrupt gates ?
	.byte	0x8e		# Ring 0 etc
	.word	0 		# offset 31:16

.endr
idt_end:	

idt_ptr:
	.word 	idt_end - idt - 1
	.long	idt

	
	
	
